package ai.pai.lensman.service;

import android.app.Service;
import android.content.Intent;
import android.os.Binder;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;

import com.android.common.executors.ThreadExecutor;
import com.android.common.utils.LogHelper;
import com.android.common.utils.NetworkUtil;

import ai.pai.lensman.bean.PhotoBean;
import ai.pai.lensman.db.DBService;

public class UploadService extends Service implements UploadTask.OnPhotoUploadListener,Handler.Callback{

    private PhotoBean currentPhoto;
    private PhotoUploadListener listener;
    private Handler handler;
    private static final int MSG_TRY_UPLOAD = 4567;

    private static final String TAG = "UploadService";

    @Override
    public void onCreate() {
        super.onCreate();
        handler = new Handler(this);
    }

    @Override
    public IBinder onBind(Intent intent) {
        return new MyBinder();
    }

    @Override
    public int onStartCommand(Intent intent, int flags, int startId) {
        if(intent!=null && intent.getSerializableExtra("photo")!=null){
            PhotoBean bean = (PhotoBean) intent.getSerializableExtra("photo");
            bean.uploadStatus = PhotoBean.UploadStatus.STATUS_NO_BEGIN;
            DBService.getInstance().updatePhotoBean(bean);
        }
            startUpload();
        return super.onStartCommand(intent, flags, startId);
    }

    private synchronized void startUpload(){
        if(!NetworkUtil.isWifiConnected(this)){
            return;
        }
        if(currentPhoto!=null){
            LogHelper.d(TAG,"当前有图片正在上传"+currentPhoto);
            return;
        }
        LogHelper.d(TAG,"查询是否有可以上传的照片，时间="+System.currentTimeMillis());
         currentPhoto = DBService.getInstance().getNextUploadPhoto();
        if(currentPhoto==null){
            handler.removeMessages(MSG_TRY_UPLOAD);
            handler.sendEmptyMessageDelayed(MSG_TRY_UPLOAD,2000);
            LogHelper.d(TAG,"本地图片已全部上传");
            return;
        }
        LogHelper.d(TAG,"即将上传的照片是"+currentPhoto);
        new UploadTask(currentPhoto,this).executeOnExecutor(ThreadExecutor.getInstance().getExecutor());
    }

    @Override
    public void onPhotoUploadSucceed(PhotoBean bean) {
        if(bean.equals(currentPhoto)){
            currentPhoto.uploadStatus =  PhotoBean.UploadStatus.STATUS_SUCCESS;
            DBService.getInstance().updatePhotoBean(currentPhoto);
            if(listener!=null){
                listener.onPhotoUploaded(currentPhoto);
            }
            currentPhoto = null;
        }
        startUpload();
    }

    @Override
    public void onPhotoUploadFail(PhotoBean bean) {
        if(bean.equals(currentPhoto)){
            currentPhoto.uploadStatus =  PhotoBean.UploadStatus.STATUS_ERROR;
            DBService.getInstance().updatePhotoBean(currentPhoto);
            if(listener!=null){
                listener.onPhotoUploadError(currentPhoto);
            }
            currentPhoto = null;
        }
        startUpload();
    }

    public void setPhotoUploadListener(PhotoUploadListener listener){
        this.listener = listener;
    }

    @Override
    public boolean handleMessage(Message message) {
        if(message.what == MSG_TRY_UPLOAD){
            startUpload();
            return true;
        }
        return false;
    }

    public interface PhotoUploadListener{
        void onPhotoUploaded(PhotoBean bean);
        void onPhotoUploadError(PhotoBean bean);
    }

    public class MyBinder extends Binder {

        public UploadService getService(){
            return UploadService.this;
        }

    }

}
